@@ -1,4 +1,8 @@
-__version__ = "4.1.2"
+# SPDX-FileCopyrightText: 2020 2020
+#
+# SPDX-License-Identifier: Apache-2.0
+
+__version__ = "4.1.5"
 
 import logging
 import os, time
@@ -92,6 +96,74 @@ def clean_before_build():
     os.makedirs(os.path.join(outputdir))
     logger.info("Cleaned out directory " + outputdir)
 
+def version_tuple(version_str):
+    """
+    convert string into tuple to compare version
+
+    Args:
+        version_str : raw string
+    Returns:
+        tuple : version into tupleformat
+    """
+    filled = []
+    for point in version_str.split("."):
+        filled.append(point.zfill(8))
+    return tuple(filled)
+
+def handle_update(config_path):
+    """
+    handle changes in globalConfig.json
+
+    Args:
+        config_path : path to globalConfig.json
+
+    Returns:
+        dictionary : schema_content (globalConfig.json)     
+    """
+    with open(config_path, "r") as config_file:
+        schema_content = json.load(config_file)
+    # check for schemaVersion in meta, if not availble then set default 0.0.0 
+    version = schema_content.get("meta").get("schemaVersion","0.0.0")
+
+    # check for schemaVersion, if it's less than 0.0.1 then updating globalConfig.json 
+    if version_tuple(version) < version_tuple("0.0.1"):
+        ta_tabs = schema_content.get("pages").get("configuration",{}).get("tabs",{})
+
+        # check in every Account tab for biased term
+        for tab in ta_tabs:
+            conf_entitties= tab.get("entity")
+            for entity in conf_entitties:
+                entity_option = entity.get("options")
+                if entity_option and "whiteList" in entity_option:
+                    entity_option["allowList"] = entity_option.get("whiteList")
+                    del entity_option["whiteList"]
+                if entity_option and "blackList" in entity_option:
+                    entity_option["denyList"] = entity_option.get("blackList")
+                    del entity_option["blackList"]
+
+        is_inputs = ("inputs" in schema_content.get("pages"))
+        if is_inputs:
+            services = schema_content.get("pages").get("inputs",{}).get("services",{})
+            # check in every Input service for biased term
+            for service in services:
+                conf_entitties= service.get("entity")
+                for entity in conf_entitties:
+                    entity_option = entity.get("options")
+                    if entity_option and "whiteList" in entity_option:
+                        entity_option["allowList"] = entity_option.get("whiteList")
+                        del entity_option["whiteList"]
+                    if entity_option and "blackList" in entity_option:
+                        entity_option["denyList"] = entity_option.get("blackList")
+                        del entity_option["blackList"]
+
+        # set schemaVersion to 0.0.1 as updated globalConfig.json according to new update
+        schema_content["meta"]["schemaVersion"]="0.0.1"
+
+        # upadating new changes in globalConfig.json 
+        with open(config_path, "w") as config_file:
+            json.dump(schema_content,config_file, ensure_ascii=False, indent=4)
+    return schema_content
+    
 
 def copy_package_source(args, ta_name):
     """
@@ -166,31 +238,30 @@ def install_libs(path, ucc_lib_target):
             os.system(install_cmd)
             remove_files(ucc_target)
     logging.info(f"  Checking for requirements in {path}")
-    print(os.path.join(path, "lib","requirements.txt"))
     if os.path.exists(os.path.join(path,"lib", "requirements.txt")):
         logging.info(f"  Uses common requirements")    
         _install_libs(requirements=os.path.join(path, "lib","requirements.txt"), ucc_target=ucc_lib_target)
-    elif os.path.exists(os.path.join(os.path.abspath(os.path.join(args.source, os.pardir)), "requirements.txt")):
+    elif os.path.exists(os.path.join(os.path.abspath(os.path.join(path, os.pardir)), "requirements.txt")):
         logging.info(f"  Uses common requirements")    
-        _install_libs(requirements=os.path.join(os.path.abspath(os.path.join(args.source, os.pardir)), "requirements.txt"), ucc_target=ucc_lib_target)
+        _install_libs(requirements=os.path.join(os.path.abspath(os.path.join(path, os.pardir)), "requirements.txt"), ucc_target=ucc_lib_target)
     else:
         logging.info(f"  Not using common requirements")    
 
     if os.path.exists(os.path.join(path,"lib","py2", "requirements.txt")):
         logging.info(f"  Uses py2 requirements")    
         _install_libs(requirements=os.path.join(path,"lib","py2", "requirements.txt"), installer="pip2", ucc_target=os.path.join(ucc_lib_target, "py2"))
-    elif os.path.exists(os.path.join(os.path.abspath(os.path.join(args.source, os.pardir)), "requirements_py2.txt")):
+    elif os.path.exists(os.path.join(os.path.abspath(os.path.join(path, os.pardir)), "requirements_py2.txt")):
         logging.info(f"  Uses py2 requirements")    
-        _install_libs(requirements=os.path.join(os.path.abspath(os.path.join(args.source, os.pardir)), "requirements_py2.txt"), installer="pip2", ucc_target=os.path.join(ucc_lib_target, "py2"))        
+        _install_libs(requirements=os.path.join(os.path.abspath(os.path.join(path, os.pardir)), "requirements_py2.txt"), installer="pip2", ucc_target=os.path.join(ucc_lib_target, "py2"))        
     else:
         logging.info(f"  Not using py2 requirements")    
 
     if os.path.exists(os.path.join(path, "lib","py3","requirements.txt")):
         logging.info(f"  Uses py3 requirements")            
         _install_libs(requirements=os.path.join(path,"lib", "py3","requirements.txt"), ucc_target=os.path.join(ucc_lib_target, "py3"))
-    elif os.path.exists(os.path.join(os.path.abspath(os.path.join(args.source, os.pardir)), "requirements_py3.txt")):
+    elif os.path.exists(os.path.join(os.path.abspath(os.path.join(path, os.pardir)), "requirements_py3.txt")):
         logging.info(f"  Uses py3 requirements")    
-        _install_libs(requirements=os.path.join(os.path.abspath(os.path.join(args.source, os.pardir)), "requirements_py3.txt"), installer="pip3", ucc_target=os.path.join(ucc_lib_target, "py2"))        
+        _install_libs(requirements=os.path.join(os.path.abspath(os.path.join(path, os.pardir)), "requirements_py3.txt"), installer="pip3", ucc_target=os.path.join(ucc_lib_target, "py2"))        
     else:
         logging.info(f"  Not using py3 requirements")    
 
@@ -340,9 +411,9 @@ def add_modular_input(
         class_name = input_name.upper()
         description = service.get("title")
         entity = service.get("entity")
-        field_white_list = ["name", "index", "sourcetype"]
-        # filter fields in white list
-        entity = [x for x in entity if x.get("field") not in field_white_list]
+        field_allow_list = ["name", "index", "sourcetype"]
+        # filter fields in allow list
+        entity = [x for x in entity if x.get("field") not in field_allow_list]
         import_declare = "import " + import_declare_name
 
         content = j2_env.get_template("input.template").render(
@@ -491,8 +562,8 @@ def main():
     else:
         stage = version.stage[:1]
     
-    version_str = version.serialize(metadata=True)
-    version_splunk = f"{version.base}-{stage}{version.commit}"
+    version_str = version.serialize(metadata=True,style=Style.SemVer)
+    version_splunk = f"{version.base}{stage}{version.commit}"
     
     parser.add_argument(
         "--ta-version",
@@ -516,8 +587,8 @@ def main():
         if args.ta_version:
             update_ta_version(args)
 
-        with open(args.config, "r") as config_file:
-            schema_content = json.load(config_file)
+        # handle_update check schemaVersion and update globalConfig.json if required and return schema
+        schema_content = handle_update(args.config)
 
         scheme = GlobalConfigBuilderSchema(schema_content, j2_env)
         ta_name = schema_content.get("meta").get("name")
@@ -601,6 +672,8 @@ def main():
         app_config.add_section('launcher')
     if not 'id' in app_config:
         app_config.add_section('id')
+    if not 'install' in app_config:
+        app_config.add_section('install')
     if not 'package' in app_config:
         app_config.add_section('package')
     if not 'ui' in app_config:
@@ -611,7 +684,7 @@ def main():
     
     app_config['id']['version']=version_splunk
 
-    app_config['package']['build']=str(int(time.time()))
+    app_config['install']['build']=str(int(time.time()))
     app_config['package']['id']=manifest['info']['id']['name'] 
 
     app_config['ui']['label']=manifest['info']['title']
